@@ -22,15 +22,17 @@
 
 #include <sys/stat.h>
 #include <sys/types.h>
-#include <errno.h>
-#include <stdio.h>
-#include <stdlib.h>
+#include <cerrno>
+#include <cstdio>
+#include <cstdlib>
 #include <pthread.h>
 #include <unistd.h>
 #ifdef linux
 #include <sys/fsuid.h>
 #endif
 
+#include <cstring>
+
 #include "Context.h"
 #include "Cipher.h"
 #include "Mutex.h"
@@ -188,101 +190,106 @@ public:
     {
     }
 
-    ~RenameOp()
-    {
-	if(renameList)
-	{
-	    // got a bunch of decoded filenames sitting in memory..  do a little
-	    // cleanup before leaving..
-	    list<RenameEl>::iterator it;
-	    for(it = renameList->begin(); it != renameList->end(); ++it)
-	    {
-		it->oldPName.assign( it->oldPName.size(), ' ' );
-		it->newPName.assign( it->newPName.size(), ' ' );
-	    }
-	}
-    }
+    ~RenameOp();
 
     operator bool () const
     {
 	return renameList;
     }
 
-    bool apply()
-    {
-	try
-	{
-	    while(last != renameList->end())
-	    {
-		// backing store rename.
-		rDebug("renaming %s -> %s",
-			last->oldCName.c_str(), last->newCName.c_str());
-
-		// internal node rename..
-		dn->renameNode( last->oldPName.c_str(), 
-			        last->newPName.c_str() );
-
-		// rename on disk..
-		if(::rename( last->oldCName.c_str(), 
-			     last->newCName.c_str() ) == -1)
-		{
-		    rWarning("Error renaming %s: %s",
-			    last->oldCName.c_str(), strerror( errno ));
-		    dn->renameNode( last->newPName.c_str(), 
-			    last->oldPName.c_str(), false );
-		    return false;
-		}
-
-		++last;
-	    }
-
-	    return true;
-	} catch( rlog::Error &err )
-	{
-	    err.log( _RLWarningChannel );
-	    return false;
-	}
-    }
-
-    void undo()
-    {
-	rDebug("in undoRename");
-
-	if(last == renameList->begin())
-	{
-	    rDebug("nothing to undo");
-	    return; // nothing to undo
-	}
-
-	// list has to be processed backwards, otherwise we may rename
-	// directories and directory contents in the wrong order!
-	int undoCount = 0;
-	list<RenameEl>::const_iterator it = last;
-
-	while( it != renameList->begin() )
-	{
-	    --it;
-
-	    rDebug("undo: renaming %s -> %s", 
-		    it->newCName.c_str(), it->oldCName.c_str());
-
-	    ::rename( it->newCName.c_str(), it->oldCName.c_str() );
-	    try
-	    {
-		dn->renameNode( it->newPName.c_str(), 
-			        it->oldPName.c_str(), false );
-	    } catch( rlog::Error &err )
-	    {
-		err.log( _RLWarningChannel );
-		// continue on anyway...
-	    }
-	    ++undoCount;
-	};
-    
-	rWarning("Undo rename count: %i", undoCount);
-    }
+    bool apply();
+    void undo();
 };
 
+RenameOp::~RenameOp()
+{
+    if(renameList)
+    {
+        // got a bunch of decoded filenames sitting in memory..  do a little
+        // cleanup before leaving..
+        list<RenameEl>::iterator it;
+        for(it = renameList->begin(); it != renameList->end(); ++it)
+        {
+            it->oldPName.assign( it->oldPName.size(), ' ' );
+            it->newPName.assign( it->newPName.size(), ' ' );
+        }
+    }
+}
+
+bool RenameOp::apply()
+{
+    try
+    {
+        while(last != renameList->end())
+        {
+            // backing store rename.
+            rDebug("renaming %s -> %s",
+                    last->oldCName.c_str(), last->newCName.c_str());
+
+            // internal node rename..
+            dn->renameNode( last->oldPName.c_str(), 
+                            last->newPName.c_str() );
+
+            // rename on disk..
+            if(::rename( last->oldCName.c_str(), 
+                         last->newCName.c_str() ) == -1)
+            {
+                rWarning("Error renaming %s: %s",
+                        last->oldCName.c_str(), strerror( errno ));
+                dn->renameNode( last->newPName.c_str(), 
+                        last->oldPName.c_str(), false );
+                return false;
+            }
+
+            ++last;
+        }
+
+        return true;
+    } catch( rlog::Error &err )
+    {
+        err.log( _RLWarningChannel );
+        return false;
+    }
+}
+
+void RenameOp::undo()
+{
+    rDebug("in undoRename");
+
+    if(last == renameList->begin())
+    {
+        rDebug("nothing to undo");
+        return; // nothing to undo
+    }
+
+    // list has to be processed backwards, otherwise we may rename
+    // directories and directory contents in the wrong order!
+    int undoCount = 0;
+    list<RenameEl>::const_iterator it = last;
+
+    while( it != renameList->begin() )
+    {
+        --it;
+
+        rDebug("undo: renaming %s -> %s", 
+                it->newCName.c_str(), it->oldCName.c_str());
+
+        ::rename( it->newCName.c_str(), it->oldCName.c_str() );
+        try
+        {
+            dn->renameNode( it->newPName.c_str(), 
+                            it->oldPName.c_str(), false );
+        } catch( rlog::Error &err )
+        {
+            err.log( _RLWarningChannel );
+            // continue on anyway...
+        }
+        ++undoCount;
+    };
+
+    rWarning("Undo rename count: %i", undoCount);
+}
+
 DirNode::DirNode(EncFS_Context *_ctx,
 	const string &sourceDir, const shared_ptr<Config> &_config)
 {
@@ -386,13 +393,15 @@ DirTraverse DirNode::openDir(const char *plaintextPath)
     string cyName = rootDir + naming->encodePath( plaintextPath );
     //rDebug("openDir on %s", cyName.c_str() );
 
-    shared_ptr<DIR> dp( ::opendir( cyName.c_str() ), DirDeleter() );
-    if(!dp)
+    DIR *dir = ::opendir( cyName.c_str() );
+    if(dir == NULL)
     {
 	rDebug("opendir error %s", strerror(errno));
-	return DirTraverse( dp, 0, shared_ptr<NameIO>() );
+	return DirTraverse( shared_ptr<DIR>(), 0, shared_ptr<NameIO>() );
     } else
     {
+        shared_ptr<DIR> dp( dir, DirDeleter() );
+
 	uint64_t iv = 0;
 	// if we're using chained IV mode, then compute the IV at this
 	// directory level..
@@ -706,6 +715,22 @@ shared_ptr<FileNode> DirNode::renameNode( const char *from, const char *to,
     return node;
 }
 
+shared_ptr<FileNode> DirNode::directLookup( const char *path )
+{
+    return shared_ptr<FileNode>( 
+            new FileNode( this, 
+                config->fsSubVersion,
+                "unknown", path, 
+                config->cipher, config->key,
+                config->blockSize, config->blockMACBytes,
+                config->blockMACRandBytes, 
+                config->uniqueIV,
+                config->externalIVChaining,
+                config->forceDecode,
+                config->reverseEncryption,
+                config->allowHoles) );
+}
+
 shared_ptr<FileNode> DirNode::findOrCreate( const char *plainName)
 {
     shared_ptr<FileNode> node;
