@@ -46,11 +46,11 @@
 #include <sys/wait.h>
 #include <fcntl.h>
 #include <unistd.h>
-#include <ctype.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <errno.h>
-#include <string.h>
+#include <cctype>
+#include <cstdio>
+#include <cstdlib>
+#include <cerrno>
+#include <cstring>
 
 #include <iostream>
 #include <sstream>
@@ -61,18 +61,27 @@
 #include <boost/archive/xml_iarchive.hpp>
 #include <boost/archive/xml_oarchive.hpp>
 #include <boost/serialization/split_free.hpp>
+#include <boost/serialization/binary_object.hpp>
+
+// disable rlog section grouping for this file.. seems to cause problems
+#undef RLOG_SECTION
+#define RLOG_SECTION
 
 using namespace rel;
 using namespace rlog;
 using namespace std;
 using namespace gnu;
 namespace fs = boost::filesystem;
+namespace serial = boost::serialization;
 
 static const int DefaultBlockSize = 1024;
 // The maximum length of text passwords.  If longer are needed,
 // use the extpass option, as extpass can return arbitrary length binary data.
 static const int MaxPassBuf = 512;
 
+static const int NormalKDFDuration = 500; // 1/2 a second
+static const int ParanoiaKDFDuration = 3000; // 3 seconds
+
 // environment variable names for values encfs stores in the environment when
 // calling an external password program.
 static const char ENCFS_ENV_ROOTDIR[] = "encfs_root";
@@ -85,7 +94,9 @@ static const char ENCFS_ENV_STDERR[] = "encfs_stderr";
 static int V5SubVersion = 20040813; // fix MACFileIO block size issues
 static int V5SubVersionDefault = 0;
 
-const int V6SubVersion = 20080813; // switch to v6/XML, add allowHoles option
+// 20080813 was really made on 20080413 -- typo on date..
+//const int V6SubVersion = 20080813; // switch to v6/XML, add allowHoles option
+const int V6SubVersion = 20080816; // add salt and iteration count
 
 struct ConfigInfo
 {
@@ -133,17 +144,23 @@ namespace boost
             ar << make_nvp("blockMACRandBytes", cfg.blockMACRandBytes);
             ar << make_nvp("allowHoles", cfg.allowHoles);
 
-            int keyLen = cfg.keyData.length();
-            ar << make_nvp("encodedKeySize", keyLen);
-            char key[keyLen];
-            memcpy(key, cfg.keyData.data(), keyLen);
-            ar << make_nvp("encodedKeyData", make_binary_object(key, keyLen));
+            int encodedSize = cfg.keyData.size();
+            ar << make_nvp("encodedKeySize", encodedSize);
+            ar << make_nvp("encodedKeyData",
+                    serial::make_binary_object(cfg.getKeyData(), encodedSize));
+
+            // version 20080816
+            int size = cfg.salt.size();
+            ar << make_nvp("saltLen", size);
+            ar << make_nvp("saltData",
+                    serial::make_binary_object(cfg.getSaltData(), size));
+            ar << make_nvp("kdfIterations", cfg.kdfIterations);
+            ar << make_nvp("desiredKDFDuration", cfg.desiredKDFDuration);
         }
 
         template<class Archive>
         void load(Archive &ar, EncFSConfig &cfg, unsigned int version)
         {
-            (void)version;
             cfg.subVersion = version;
             ar >> make_nvp("creator", cfg.creator);
             ar >> make_nvp("cipherAlg", cfg.cipherIface);
@@ -156,13 +173,35 @@ namespace boost
             ar >> make_nvp("blockMACBytes", cfg.blockMACBytes);
             ar >> make_nvp("blockMACRandBytes", cfg.blockMACRandBytes);
             ar >> make_nvp("allowHoles", cfg.allowHoles);
-       
-            int encodedKeySize;
-            ar >> make_nvp("encodedKeySize", encodedKeySize);
-            char key[encodedKeySize];
-            ar >> make_nvp("encodedKeyData", 
-                    make_binary_object(key, encodedKeySize));
-            cfg.keyData.assign( (char*)key, encodedKeySize );
+      
+            int encodedSize;
+            ar >> make_nvp("encodedKeySize", encodedSize);
+            rAssert(encodedSize == cfg.getCipher()->encodedKeySize());
+
+            unsigned char *key = new unsigned char[encodedSize];
+            ar >> make_nvp("encodedKeyData",
+                    serial::make_binary_object(key, encodedSize));
+            cfg.assignKeyData(key, encodedSize);
+            delete[] key;
+
+            if(version >= 20080816)
+            {
+                int saltLen;
+                ar >> make_nvp("saltLen", saltLen);
+                unsigned char *salt = new unsigned char[saltLen];
+                ar >> make_nvp("saltData",
+                        serial::make_binary_object(salt, saltLen));
+                cfg.assignSaltData(salt, saltLen);
+                delete[] salt;
+
+                ar >> make_nvp("kdfIterations", cfg.kdfIterations);
+                ar >> make_nvp("desiredKDFDuration", cfg.desiredKDFDuration);
+            } else
+            {
+                cfg.salt.clear();
+                cfg.kdfIterations = 16;
+                cfg.desiredKDFDuration = NormalKDFDuration;
+            }
         }
         
         template<class Archive>
@@ -275,7 +314,10 @@ ConfigType readConfig_load( ConfigInfo *nm, const char *path,
 	try
 	{
 	    if( (*nm->loadFunc)( path, config, nm ))
+            {
+                config->cfgType = nm->type;
 		return nm->type;
+            }
 	} catch( rlog::Error & err )
 	{
 	    err.log( _RLWarningChannel );
@@ -286,6 +328,7 @@ ConfigType readConfig_load( ConfigInfo *nm, const char *path,
     } else
     {
 	// No load function - must be an unsupported type..
+        config->cfgType = nm->type;
 	return nm->type;
     }
 }
@@ -318,7 +361,7 @@ bool readV6Config( const char *configFile, EncFSConfig *config,
 {
     (void)info;
 
-    fs::ifstream st(configFile);
+    fs::ifstream st( configFile );
     if(st.is_open())
     {
         try
@@ -373,7 +416,10 @@ bool readV5Config( const char *configFile, EncFSConfig *config,
 	    cfgRdr["naming"] >> config->nameIface;
 	    cfgRdr["keySize"] >> config->keySize;
 	    cfgRdr["blockSize"] >> config->blockSize;
-	    cfgRdr["keyData"] >> config->keyData;
+
+            string data;
+	    cfgRdr["keyData"] >> data;
+            config->assignKeyData(data);
 	    config->uniqueIV = cfgRdr["uniqueIV"].readBool( false );
 	    config->chainedNameIV = cfgRdr["chainedIV"].readBool( false );
 	    config->externalIVChaining = cfgRdr["externalIV"].readBool( false );
@@ -407,7 +453,9 @@ bool readV4Config( const char *configFile, EncFSConfig *config,
 	    cfgRdr["cipher"] >> config->cipherIface;
 	    cfgRdr["keySize"] >> config->keySize;
 	    cfgRdr["blockSize"] >> config->blockSize;
-	    cfgRdr["keyData"] >> config->keyData;
+            string data;
+	    cfgRdr["keyData"] >> data;
+            config->assignKeyData(data);
 
 	    // fill in default for V4
 	    config->nameIface = Interface("nameio/stream", 1, 0, 0);
@@ -478,6 +526,22 @@ bool writeV6Config( const char *configFile, EncFSConfig *config )
     return true;
 }
 
+std::ostream &operator << (std::ostream &st, const EncFSConfig &cfg)
+{
+    boost::archive::xml_oarchive oa(st);
+    oa << BOOST_SERIALIZATION_NVP( cfg );
+
+    return st;
+}
+
+std::istream &operator >> (std::istream &st, EncFSConfig &cfg)
+{
+    boost::archive::xml_iarchive ia(st);
+    ia >> BOOST_SERIALIZATION_NVP( cfg );
+
+    return st;
+}
+
 bool writeV5Config( const char *configFile, EncFSConfig *config )
 {
     ConfigReader cfg;
@@ -488,7 +552,9 @@ bool writeV5Config( const char *configFile, EncFSConfig *config )
     cfg["naming"] << config->nameIface;
     cfg["keySize"] << config->keySize;
     cfg["blockSize"] << config->blockSize;
-    cfg["keyData"] << config->keyData;
+    string key;
+    key.assign((char *)config->getKeyData(), config->keyData.size());
+    cfg["keyData"] << key;
     cfg["blockMACBytes"] << config->blockMACBytes;
     cfg["blockMACRandBytes"] << config->blockMACRandBytes;
     cfg["uniqueIV"] << config->uniqueIV;
@@ -505,7 +571,9 @@ bool writeV4Config( const char *configFile, EncFSConfig *config )
     cfg["cipher"] << config->cipherIface;
     cfg["keySize"] << config->keySize;
     cfg["blockSize"] << config->blockSize;
-    cfg["keyData"] << config->keyData;
+    string key;
+    key.assign((char *)config->getKeyData(), config->keyData.size());
+    cfg["keyData"] << key;
 
     return cfg.save( configFile );
 }
@@ -754,10 +822,10 @@ bool boolDefaultNo(const char *prompt)
     fgets( answer, sizeof(answer), stdin );
     cout << "\n";
 
-    if(tolower(answer[0]) == 'n')
-	return false;
-    else
+    if(tolower(answer[0]) == 'y')
 	return true;
+    else
+	return false;
 }
 
 static 
@@ -814,10 +882,10 @@ bool boolDefaultYes(const char *prompt)
     fgets( answer, sizeof(answer), stdin );
     cout << "\n";
 
-    if(tolower(answer[0]) == 'y')
-	return true;
-    else
+    if(tolower(answer[0]) == 'n')
 	return false;
+    else
+	return true;
 }
 
 static 
@@ -858,7 +926,7 @@ static
 bool selectZeroBlockPassThrough()
 {
     // xgroup(setup)
-    return boolDefaultNo(
+    return boolDefaultYes(
             _("Enable file-hole pass-through?\n"
 	"This avoids writing encrypted blocks when file holes are created."));
 }
@@ -895,7 +963,8 @@ RootPtr createV6Config( EncFS_Context *ctx, const std::string &rootDir,
     bool uniqueIV = false;
     bool chainedIV = false;
     bool externalIV = false;
-    bool allowHoles = false;
+    bool allowHoles = true;
+    long desiredKDFDuration = NormalKDFDuration;
     
     if (reverseEncryption)
     {
@@ -930,6 +999,7 @@ RootPtr createV6Config( EncFS_Context *ctx, const std::string &rootDir,
 	uniqueIV = true;
 	chainedIV = true;
 	externalIV = true;
+        desiredKDFDuration = ParanoiaKDFDuration;
     } else
     if(answer[0] != 'x')
     {
@@ -1011,6 +1081,7 @@ RootPtr createV6Config( EncFS_Context *ctx, const std::string &rootDir,
     
     EncFSConfig config;
 
+    config.cfgType = Config_V6;
     config.cipherIface = cipher->interface();
     config.keySize = keySize;
     config.blockSize = blockSize;
@@ -1024,6 +1095,10 @@ RootPtr createV6Config( EncFS_Context *ctx, const std::string &rootDir,
     config.externalIVChaining = externalIV;
     config.allowHoles = allowHoles;
 
+    config.salt.clear();
+    config.kdfIterations = 0; // filled in by keying function
+    config.desiredKDFDuration = desiredKDFDuration;
+
     cout << "\n";
     // xgroup(setup)
     cout << _("Configuration finished.  The filesystem to be created has\n"
@@ -1060,16 +1135,16 @@ RootPtr createV6Config( EncFS_Context *ctx, const std::string &rootDir,
     CipherKey userKey;
     rDebug( "useStdin: %i", useStdin );
     if(useStdin)
-	userKey = getUserKey( cipher, useStdin );
-    else if(passwordProgram.empty())
-	userKey = getNewUserKey( cipher );
+	userKey = config.getUserKey( useStdin );
+    else if(!passwordProgram.empty())
+	userKey = config.getUserKey( passwordProgram, rootDir );
     else
-	userKey = getUserKey( passwordProgram, cipher, rootDir );
+	userKey = config.getNewUserKey();
 
     cipher->writeKey( volumeKey, encodedKey, userKey );
     userKey.reset();
 
-    config.keyData.assign( (char*)encodedKey, encodedKeySize );
+    config.assignKeyData(encodedKey, encodedKeySize);
     delete[] encodedKey;
 
     if(!volumeKey)
@@ -1172,7 +1247,7 @@ void showFSInfo( const EncFSConfig &config )
     }
     {
 	cout << autosprintf(_("Key Size: %i bits"), config.keySize);
-	cipher = Cipher::New( config.cipherIface, config.keySize );
+	cipher = config.getCipher();
 	if(!cipher)
 	{
 	    // xgroup(diag)
@@ -1180,6 +1255,13 @@ void showFSInfo( const EncFSConfig &config )
 	} else
 	    cout << "\n";
     }
+    if(config.kdfIterations > 0 && config.salt.size() > 0)
+    {
+	cout << autosprintf(_("Using PBKDF2, with %i iterations"), 
+                              config.kdfIterations) << "\n";
+	cout << autosprintf(_("Salt Size: %i bits"), 
+                8*(int)config.salt.size()) << "\n";
+    }
     if(config.blockMACBytes)
     {
 	if(config.subVersion < 20040813)
@@ -1227,8 +1309,79 @@ void showFSInfo( const EncFSConfig &config )
     }
     cout << "\n";
 }
-    
-CipherKey getUserKey( const shared_ptr<Cipher> &cipher, bool useStdin )
+   
+shared_ptr<Cipher> EncFSConfig::getCipher() const
+{
+    return Cipher::New( cipherIface, keySize );
+}
+
+void EncFSConfig::assignKeyData(const std::string &in)
+{
+    keyData.resize(in.length());
+    for(unsigned int i=0; i<in.length(); ++i)
+        keyData[i] = in.data()[i];
+}
+
+void EncFSConfig::assignKeyData(unsigned char *data, int len)
+{
+    keyData.resize(len);
+    for(int i=0; i<len; ++i)
+        keyData[i] = data[i];
+}
+
+void EncFSConfig::assignSaltData(unsigned char *data, int len)
+{
+    salt.resize(len);
+    for(int i=0; i<len; ++i)
+        salt[i] = data[i];
+}
+
+unsigned char *EncFSConfig::getKeyData() const
+{
+    return const_cast<unsigned char *>(&keyData.front());
+}
+
+unsigned char *EncFSConfig::getSaltData() const
+{
+    return const_cast<unsigned char *>(&salt.front());
+}
+
+CipherKey EncFSConfig::makeKey(const char *password, int passwdLen)
+{
+    CipherKey userKey;
+    shared_ptr<Cipher> cipher = getCipher();
+
+    // if no salt is set and we're creating a new password for a new
+    // FS type, then initialize salt..
+    if(salt.size() == 0 && kdfIterations == 0 && cfgType >= Config_V6)
+    {
+        // upgrade to using salt
+        salt.resize(20);
+    }
+
+    if(salt.size() > 0)
+    {
+        // if iterations isn't known, then we're creating a new key, so
+        // randomize the salt..
+        if(kdfIterations == 0 && !cipher->randomize( 
+                    getSaltData(), salt.size(), true))
+        {
+            cout << _("Error creating salt\n");
+            return userKey;
+        }
+
+        userKey = cipher->newKey( password, passwdLen,
+                kdfIterations, desiredKDFDuration, 
+                getSaltData(), salt.size());
+    } else
+    {
+        userKey = cipher->newKey( password, passwdLen );
+    }
+
+    return userKey;
+}
+
+CipherKey EncFSConfig::getUserKey(bool useStdin)
 {
     char passBuf[MaxPassBuf];
     char *res;
@@ -1250,7 +1403,7 @@ CipherKey getUserKey( const shared_ptr<Cipher> &cipher, bool useStdin )
     if(!res)
 	cerr << _("Zero length password not allowed\n");
     else
-	userKey = cipher->newKey( passBuf, strlen(passBuf) );
+        userKey = makeKey(passBuf, strlen(passBuf));
 
     memset( passBuf, 0, sizeof(passBuf) );
 
@@ -1283,8 +1436,8 @@ std::string readPassword( int FD )
     return result;
 }
 
-CipherKey getUserKey( const std::string &passProg, 
-	const shared_ptr<Cipher> &cipher, const std::string &rootDir )
+CipherKey EncFSConfig::getUserKey( const std::string &passProg,
+        const std::string &rootDir )
 {
     // have a child process run the command and get the result back to us.
     int fds[2], pid;
@@ -1355,7 +1508,7 @@ CipherKey getUserKey( const std::string &passProg,
     waitpid(pid, NULL, 0);
 
     // convert to key..
-    result = cipher->newKey( password.c_str(), password.length() );
+    result = makeKey(password.c_str(), password.length());
 
     // clear buffer..
     password.assign( password.length(), '\0' );
@@ -1363,7 +1516,7 @@ CipherKey getUserKey( const std::string &passProg,
     return result;
 }
 
-CipherKey getNewUserKey( const shared_ptr<Cipher> &cipher )
+CipherKey EncFSConfig::getNewUserKey()
 {
     CipherKey userKey;
     char passBuf[MaxPassBuf];
@@ -1379,8 +1532,9 @@ CipherKey getNewUserKey( const shared_ptr<Cipher> &cipher )
 		sizeof(passBuf2)-1, RPP_ECHO_OFF);
 
 	if(res1 && res2 && !strcmp(passBuf, passBuf2))
-	    userKey = cipher->newKey( passBuf, strlen(passBuf) );
-	else
+        {
+            userKey = makeKey(passBuf, strlen(passBuf));
+        } else
 	{
 	    // xgroup(common) -- probably not common, but group with the others
 	    cerr << _("Passwords did not match, please try again\n");
@@ -1412,8 +1566,7 @@ RootPtr initFS( EncFS_Context *ctx, const shared_ptr<EncFS_Opts> &opts )
 	}
 
 // first, instanciate the cipher.
-	shared_ptr<Cipher> cipher = 
-	    Cipher::New( config.cipherIface, config.keySize );
+	shared_ptr<Cipher> cipher = config.getCipher();
 	if(!cipher)
 	{
 	    rError(_("Unable to find cipher %s, version %i:%i:%i"),
@@ -1428,23 +1581,21 @@ RootPtr initFS( EncFS_Context *ctx, const shared_ptr<EncFS_Opts> &opts )
 
 	// get user key
 	CipherKey userKey;
-	rDebug( "useStdin: %i", opts->useStdin );
-	if(opts->passwordProgram.empty())
-	    userKey = getUserKey( cipher, opts->useStdin );
-	else
-	    userKey = getUserKey( opts->passwordProgram, 
-		    cipher, opts->rootDir );
+
+        if(opts->passwordProgram.empty())
+        {
+            rDebug( "useStdin: %i", opts->useStdin );
+            userKey = config.getUserKey( opts->useStdin );
+        } else
+            userKey = config.getUserKey( opts->passwordProgram, opts->rootDir );
 
 	if(!userKey)
 	    return rootInfo;
 
-	// no funny stuff
-	rDebug("configuration key size = %i", (int)config.keyData.length());
 	rDebug("cipher key size = %i", cipher->encodedKeySize());
-	rAssert( (int)config.keyData.length() == cipher->encodedKeySize() ); 
 	// decode volume key..
-	CipherKey volumeKey = cipher->readKey( 
-		(unsigned char*)config.keyData.data(), userKey, opts->checkKey);
+	CipherKey volumeKey = cipher->readKey(
+                config.getKeyData(), userKey, opts->checkKey);
 	userKey.reset();
 
 	if(!volumeKey)
