@@ -38,8 +38,20 @@ std::string parentDirectory( const std::string &path );
 // do it and return true.
 bool userAllowMkdir( const char *dirPath, mode_t mode );
 
+enum ConfigType
+{
+    Config_None = 0,
+    Config_Prehistoric,
+    Config_V3,
+    Config_V4,
+    Config_V5,
+    Config_V6
+};
+
 struct EncFSConfig
 {
+    ConfigType cfgType;
+
     std::string creator;
     int subVersion;
 
@@ -49,7 +61,12 @@ struct EncFSConfig
     rel::Interface nameIface;
     int keySize; // reported in bits
     int blockSize; // reported in bytes
-    std::string keyData;
+
+    std::vector<unsigned char> keyData;
+
+    std::vector<unsigned char> salt;
+    int kdfIterations;
+    long desiredKDFDuration;
 
     int blockMACBytes; // MAC headers on blocks..
     int blockMACRandBytes; // number of random bytes in the block header
@@ -61,7 +78,10 @@ struct EncFSConfig
     bool allowHoles; // allow holes in files (implicit zero blocks)
 
     EncFSConfig()
+        : keyData()
+        , salt()
     {
+        cfgType = Config_None;
         subVersion = 0;
         blockMACBytes = 0;
         blockMACRandBytes = 0;
@@ -69,18 +89,32 @@ struct EncFSConfig
         externalIVChaining = false;
         chainedNameIV = false;
         allowHoles = false;
-    }
-};
 
-enum ConfigType
-{
-    Config_None = 0,
-    Config_Prehistoric,
-    Config_V3,
-    Config_V4,
-    Config_V5,
-    Config_V6
+        kdfIterations = 0;
+        desiredKDFDuration = 500;
+    }
+
+    CipherKey getUserKey(bool useStdin);
+    CipherKey getUserKey(const std::string &passwordProgram,
+                         const std::string &rootDir);
+    CipherKey getNewUserKey();
+    
+    shared_ptr<Cipher> getCipher() const;
+
+    // deprecated
+    void assignKeyData(const std::string &in);
+    void assignKeyData(unsigned char *data, int length);
+    void assignSaltData(unsigned char *data, int length);
+
+    unsigned char *getKeyData() const;
+    unsigned char *getSaltData() const;
+
+private:
+    CipherKey makeKey(const char *password, int passwdLen);
 };
+    
+std::ostream &operator << (std::ostream &os, const EncFSConfig &cfg);
+std::istream &operator >> (std::istream &os, EncFSConfig &cfg);
 
 class Cipher;
 
@@ -163,12 +197,4 @@ bool readV6Config( const char *configFile, EncFSConfig *config,
 	struct ConfigInfo *);
 bool writeV6Config( const char *configFile, EncFSConfig *config);
 
-
-
-CipherKey getUserKey(const boost::shared_ptr<Cipher> &cipher, bool useStdin);
-CipherKey getUserKey(const std::string &passwordProgram, 
-	             const boost::shared_ptr<Cipher> &cipher,
-		     const std::string &rootDir );
-CipherKey getNewUserKey(const boost::shared_ptr<Cipher> &cipher);
-
 #endif
