@@ -31,9 +31,11 @@
 #include "MemoryPool.h"
 #include "Mutex.h"
 
-#include <string.h>
+#include <cstring>
+#include <ctime>
 
 #include <sys/mman.h>
+#include <sys/time.h>
 
 #include <rlog/rlog.h>
 #include <rlog/Error.h>
@@ -125,13 +127,53 @@ int BytesToKey( int keyLen, int ivLen, const EVP_MD *md,
     return keyLen;
 }
 
+long time_diff(const timeval &end, const timeval &start)
+{
+    return (end.tv_sec - start.tv_sec) * 1000 * 1000 +
+        (end.tv_usec - start.tv_usec);
+}
+
+int TimedPBKDF2(const char *pass, int passlen,
+        const unsigned char *salt, int saltlen,
+        int keylen, unsigned char *out,
+        long desiredPDFTime)
+{
+    int iter = 1000;
+    timeval start, end;
+
+    for(;;)
+    {
+        gettimeofday( &start, 0 );
+        int res = PKCS5_PBKDF2_HMAC_SHA1(pass, passlen, salt, saltlen, 
+                                    iter, keylen, out);
+        if(res != 1)
+            return -1;
+
+        gettimeofday( &end, 0 );
+
+        long delta = time_diff(end, start);
+        if(delta < desiredPDFTime / 8)
+        {
+            iter *= 4;
+        } else if(delta < (5 * desiredPDFTime / 6))
+        {   
+            // estimate number of iterations to get close to desired time
+            iter = (int)((double)iter * (double)desiredPDFTime 
+                    / (double)delta);
+        } else
+            return iter;
+    }
+}
+
+
 // - Version 1:0 used EVP_BytesToKey, which didn't do the right thing for
 // Blowfish key lengths > 128 bit.
 // - Version 2:0 uses BytesToKey. 
 // We support both 2:0 and 1:0, hence current:revision:age = 2:0:1
 // - Version 2:1 adds support for Message Digest function interface
-static Interface BlowfishInterface( "ssl/blowfish", 2, 1, 1 );
-static Interface AESInterface( "ssl/aes", 2, 1, 1 );
+// - Version 2:2 adds PBKDF2 for password derivation
+static Interface BlowfishInterface( "ssl/blowfish", 2, 2, 1 );
+static Interface AESInterface( "ssl/aes", 2, 2, 1 );
 
 #if defined(HAVE_EVP_BF)
 
@@ -210,8 +252,8 @@ public:
     unsigned int keySize; // in bytes
     unsigned int ivLength;
 
-    // key data is first _keySize bytes, followed by _ivLength length bytes for
-    // iv
+    // key data is first _keySize bytes, 
+    // followed by iv of _ivLength bytes,
     unsigned char *buffer; 
 
     EVP_CIPHER_CTX block_enc;
@@ -232,6 +274,7 @@ SSLKey::SSLKey(int keySize_, int ivLength_)
     pthread_mutex_init( &mutex, 0 );
     buffer = (unsigned char *)OPENSSL_malloc( keySize + ivLength );
     memset( buffer, 0, keySize + ivLength );
+
     // most likely fails unless we're running as root, or a user-page-lock
     // kernel patch is applied..
     mlock( buffer, keySize + ivLength );
@@ -296,7 +339,7 @@ void initKey(const shared_ptr<SSLKey> &key, const EVP_CIPHER *_blockCipher,
 
     EVP_EncryptInit_ex( &key->block_enc, NULL, NULL, KeyData(key), NULL);
     EVP_DecryptInit_ex( &key->block_dec, NULL, NULL, KeyData(key), NULL);
-    EVP_DecryptInit_ex( &key->stream_enc, NULL, NULL, KeyData(key), NULL);
+    EVP_EncryptInit_ex( &key->stream_enc, NULL, NULL, KeyData(key), NULL);
     EVP_DecryptInit_ex( &key->stream_dec, NULL, NULL, KeyData(key), NULL);
     
     HMAC_CTX_init( &key->mac_ctx );
@@ -351,6 +394,49 @@ Interface SSL_Cipher::interface() const
     This algorithm must remain constant for backward compatibility, as this key
     is used to encipher/decipher the master key.
 */
+CipherKey SSL_Cipher::newKey(const char *password, int passwdLength,
+        int &iterationCount, long desiredDuration,
+        const unsigned char *salt, int saltLen)
+{
+    const EVP_MD *md = EVP_sha1();
+    if(!md)
+    {
+	rError("Unknown digest SHA1");
+	return CipherKey();
+    }
+   
+    shared_ptr<SSLKey> key( new SSLKey( _keySize, _ivLength) );
+    
+    if(iterationCount == 0)
+    {
+        // timed run, fills in iteration count
+        int res = TimedPBKDF2(password, passwdLength, 
+                    salt, saltLen,
+                    _keySize+_ivLength, KeyData(key),
+                    1000 * desiredDuration);
+        if(res <= 0)
+        {
+            rWarning("openssl error, PBKDF2 failed");
+            return CipherKey();
+        } else
+            iterationCount = res;
+    } else
+    {
+        // known iteration length
+        if(PKCS5_PBKDF2_HMAC_SHA1(password, passwdLength, salt, saltLen, 
+                    iterationCount, _keySize + _ivLength, 
+                    KeyData(key)) != 1)
+        {
+            rWarning("openssl error, PBKDF2 failed");
+            return CipherKey();
+        }
+    }
+  
+    initKey( key, _blockCipher, _streamCipher, _keySize );
+
+    return key;
+}
+
 CipherKey SSL_Cipher::newKey(const char *password, int passwdLength)
 {
     const EVP_MD *md = EVP_sha1();
@@ -402,32 +488,25 @@ CipherKey SSL_Cipher::newRandomKey()
 {
     const int bufLen = MAX_KEYLENGTH;
     unsigned char tmpBuf[ bufLen ];
-    // to avoid warnings of uninitialized data from valgrind
-    memset(tmpBuf, 0, sizeof(tmpBuf)); 
-    if(RAND_bytes( tmpBuf, bufLen ) == 0)
-    {
-	char errStr[120]; // specs require string at least 120 bytes long..
-	unsigned long errVal = 0;
-	if((errVal = ERR_get_error()) != 0)
-	{
-	    rWarning("openssl error: %s", ERR_error_string( errVal, errStr ));
-	    return CipherKey();
-	}
-    }
+    int saltLen = 20;
+    unsigned char saltBuf[ saltLen ];
+
+    if(!randomize(tmpBuf, bufLen, true) ||
+       !randomize(saltBuf, saltLen, true))
+        return CipherKey();
 
     shared_ptr<SSLKey> key( new SSLKey( _keySize, _ivLength) );
 
     // doesn't need to be versioned, because a random key is a random key..
     // Doesn't need to be reproducable..
-    int bytes = BytesToKey( _keySize, _ivLength, EVP_sha1(), tmpBuf, 
-	    bufLen, 16, KeyData(key), IVData(key) );
-    if(bytes != (int)_keySize)
+    if(PKCS5_PBKDF2_HMAC_SHA1((char*)tmpBuf, bufLen, saltBuf, saltLen, 
+            1000, _keySize + _ivLength, KeyData(key)) != 1)
     {
-	rWarning("newKey: BytesToKey returned %i, expecting %i key bytes",
-		bytes, _keySize);
+        rWarning("openssl error, PBKDF2 failed");
+        return CipherKey();
     }
 
-    memset( tmpBuf, 0, bufLen );
+    OPENSSL_cleanse(tmpBuf, bufLen);
 
     initKey( key, _blockCipher, _streamCipher, _keySize );
 
@@ -478,11 +557,27 @@ static uint64_t _checksum_64( SSLKey *key,
     return value;
 }
 
-void SSL_Cipher::randomize( unsigned char *buf, int len ) const
+bool SSL_Cipher::randomize( unsigned char *buf, int len,
+        bool strongRandom ) const
 {
+    // to avoid warnings of uninitialized data from valgrind
     memset(buf, 0, len); 
-    int result = RAND_pseudo_bytes( buf, len );
-    rAssert( result >= 0 );
+    int result;
+    if(strongRandom)
+        result = RAND_bytes( buf, len );
+    else
+        result = RAND_pseudo_bytes( buf, len );
+        
+    if(result != 1)
+    {
+	char errStr[120]; // specs require string at least 120 bytes long..
+	unsigned long errVal = 0;
+	if((errVal = ERR_get_error()) != 0)
+	    rWarning("openssl error: %s", ERR_error_string( errVal, errStr ));
+
+        return false;
+    } else
+        return true;
 }
 
 uint64_t SSL_Cipher::MAC_64( const unsigned char *data, int len,
