@@ -65,6 +65,7 @@ static int cmd_encode( int argc, char **argv );
 static int cmd_showcruft( int argc, char **argv );
 static int cmd_cat( int argc, char **argv );
 static int cmd_export( int argc, char **argv );
+static int cmd_showKey( int argc, char **argv );
 
 struct CommandOpts
 {
@@ -79,6 +80,9 @@ struct CommandOpts
     {"info", 1, 1, showInfo, "(root dir)", 
 	// xgroup(usage)
 	gettext_noop("  -- show information (Default command)")},
+    {"showKey", 1, 1, cmd_showKey, "(root dir)", 
+	// xgroup(usage)
+	gettext_noop("  -- show key")},
     {"passwd", 1, 1, chpasswd, "(root dir)",
 	// xgroup(usage)
 	gettext_noop("  -- change password for volume")},
@@ -233,6 +237,24 @@ static RootPtr initRootInfo(const char* crootDir)
     return result;
 }
 
+static int cmd_showKey( int argc, char **argv )
+{
+    RootPtr rootInfo = initRootInfo(argv[1]);
+    
+    if(!rootInfo)
+        return EXIT_FAILURE;
+    else
+    {
+        // encode with itself
+        string b64Key = rootInfo->cipher->encodeAsString(
+                rootInfo->volumeKey, rootInfo->volumeKey );
+
+        cout << b64Key << "\n";
+
+        return EXIT_SUCCESS;
+    }
+}
+
 static int cmd_decode( int argc, char **argv )
 {
     RootPtr rootInfo = initRootInfo(argv[1]);
@@ -334,6 +356,19 @@ int processContents( const shared_ptr<EncFS_Root> &rootInfo,
     int errCode = 0;
     shared_ptr<FileNode> node = rootInfo->root->openNode( path, "encfsctl",
 	    O_RDONLY, &errCode );
+
+    if(!node)
+    {
+        // try opening directly, so a cipher-path can be passed in
+        node = rootInfo->root->directLookup( path );
+        if(node)
+        {
+            errCode = node->open( O_RDONLY );
+            if(errCode < 0)
+                node.reset();
+        }
+    }
+
     if(!node)
     {
 	cerr << "unable to open " << path << "\n";
@@ -634,17 +669,13 @@ static int do_chpasswd( bool useStdin, int argc, char **argv )
 
     // ask for existing password
     cout << _("Enter current Encfs password\n");
-    CipherKey userKey = getUserKey( cipher, useStdin );
+    CipherKey userKey = config.getUserKey( useStdin );
     if(!userKey)
 	return EXIT_FAILURE;
 
-    rAssert( (int)config.keyData.length() == cipher->encodedKeySize() );
-
-
     // decode volume key using user key -- at this point we detect an incorrect
     // password if the key checksum does not match (causing readKey to fail).
-    CipherKey volumeKey = cipher->readKey(
-	    (unsigned char*)config.keyData.data(), userKey );
+    CipherKey volumeKey = cipher->readKey( config.getKeyData(), userKey );
 
     if(!volumeKey)
     {
@@ -655,10 +686,13 @@ static int do_chpasswd( bool useStdin, int argc, char **argv )
     // Now, get New user key..
     userKey.reset();
     cout << _("Enter new Encfs password\n");
+    // reinitialize salt and iteration count
+    config.kdfIterations = 0; // generate new
+
     if( useStdin )
-	userKey = getUserKey( cipher, true );
+	userKey = config.getUserKey( true );
     else
-	userKey = getNewUserKey( cipher );
+	userKey = config.getNewUserKey();
 
     // re-encode the volume key using the new user key and write it out..
     int result = EXIT_FAILURE;
@@ -671,8 +705,8 @@ static int do_chpasswd( bool useStdin, int argc, char **argv )
 	cipher->writeKey( volumeKey, keyBuf, userKey );
 	userKey.reset();
 
-	config.keyData.assign( (char*)keyBuf, encodedKeySize );
-	delete[] keyBuf;
+        config.assignKeyData( keyBuf, encodedKeySize );
+        delete[] keyBuf;
 
 	if(saveConfig( cfgType, rootDir, &config ))
 	{
@@ -683,6 +717,9 @@ static int do_chpasswd( bool useStdin, int argc, char **argv )
 	{
 	    cout << _("Error saving modified config file.\n");
 	}
+    } else
+    {
+        cout << _("Error creating key\n");
     }
 
     volumeKey.reset();
